/*@
  theory {
    // sum(a,m,n) = a[m]+...+a[n]
    sum: (ARRAY Base.int OF Base.int, INT, INT) -> INT;
    sumaxiom: AXIOM
      FORALL(a: ARRAY Base.int OF Base.int, m: INT, n: INT):
        IF m > n THEN 
          sum(a, m, n) = 0
        ELSE 
          (Base.MIN_INT <= n AND n <= Base.MAX_INT) =>
            sum(a, m, n) = sum(a, m, n-1)+a[n]
        ENDIF;
   }
@*/
public class MinSum
{
  // returns a position r in a such that a[0]+...+a[r] is minimal.
  public static int minSum(int[] a)
  /*@
      requires 
        (VAR a).null = FALSE;
      ensures
        LET n = (VAR a).length, r = VALUE@NEXT IN
        IF n = 0 THEN
          r = -1
        ELSE
          0 <= r AND r < n AND
          (FORALL(s: INT):
             0 <= s AND s < n => 
               sum((VAR a).value, 0, r) <= sum((VAR a).value, 0, s))
        ENDIF;
    @*/
  {
    int n = a.length;
    if (n == 0)
      return -1;
    else
    {
      int m = a[0];
      int r = 0;
      for (int i=1; i<n; i++)
      /*@
          invariant
            (VAR a).null = FALSE AND VAR n = (VAR a).length AND
            1 <= VAR i AND VAR i <= VAR n AND
            0 <= VAR r AND VAR r < VAR i AND
            VAR m = sum((VAR a).value, 0, VAR r) AND
            (FORALL(s: INT):
              (0 <= s AND s < VAR i) =>
              (VAR m <= sum((VAR a).value, 0, s)));
          decreases (VAR n)-(VAR i);
        @*/
      {
        int m0 = sum(a, i);
        if (m0 < m) 
        {
          m = m0;
          r = i;
        }
      }
      return r;
    }
  }
  
  // returns a[0]+...+a[i]
  public static int sum(int[] a, int i)
  /*@
      requires (VAR a).null = FALSE AND 
        0 <= (VAR i) AND (VAR i) < (VAR a).length;
      ensures VALUE@NEXT = sum((VAR a).value, 0, VAR i);
    @*/
  {
    int s = 0;
    for (int j=0; j<=i; j++)
    /*@
        invariant 
          (VAR a).null = FALSE AND 
          0 <= (VAR i) AND (VAR i) < (VAR a).length AND
          0 <= VAR j AND VAR j <= (VAR i)+1 AND
          VAR s = sum((VAR a).value, 0, (VAR j)-1);
        decreases (VAR i)-(VAR j)+1;
      @*/
      s = s+a[j];
    return s;
  }
}

